@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright © 2012-2013, marmuta
+# Copyright © 2012-2016 marmuta <marmvta@gmail.com>
 #
 # This file is part of Onboard.
 #
@@ -29,9 +29,12 @@ from __future__ import division, print_function, unicode_literals
 import time
 import copy
 
+from Onboard.Version import require_gi_versions
+require_gi_versions()
 from gi.repository         import Gdk
 
-from Onboard.utils         import Timer, EventSource
+from Onboard.utils         import EventSource
+from Onboard.Timer         import Timer
 from Onboard.definitions   import TouchInputEnum
 from Onboard.XInput        import XIDeviceManager, XIEventType, XIEventMask, \
                                   XIDeviceEventLogger
