@@ -1,5 +1,26 @@
 #!/usr/bin/python3
 # -*- coding: utf-8 -*-
+
+# Copyright © 2006-2007, 2009 Chris Jones <tortoise@tortuga>
+# Copyright © 2008-2010 Francesco Fumanti <francesco.fumanti@gmx.net>
+# Copyright © 2011-2012 Gerd Kohlberger <lowfi@chello.at>
+# Copyright © 2011-2016 marmuta <marmvta@gmail.com>
+#
+# This file is part of Onboard.
+#
+# Onboard is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 3 of the License, or
+# (at your option) any later version.
+#
+# Onboard is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
+
 """ Onboard preferences utility """
 
 from __future__ import division, print_function, unicode_literals
@@ -15,18 +36,19 @@ try:
 except ImportError:
     pass
 
+from Onboard.Version import require_gi_versions
+require_gi_versions()
 from gi.repository import GObject, Pango, Gdk, Gtk
 
-# install translation function _() for all modules
 from Onboard.LayoutLoaderSVG import LayoutLoaderSVG
 from Onboard.SnippetView     import SnippetView
 from Onboard.Appearance      import Theme, ColorScheme
 from Onboard.Scanner         import ScanMode, ScanDevice
 from Onboard.XInput          import XIDeviceManager, XIEventType
-from Onboard.utils           import show_ask_string_dialog, \
-                                    show_confirmation_dialog, \
-                                    unicode_str, open_utf8, escape_markup, \
+from Onboard.utils           import unicode_str, open_utf8, escape_markup, \
                                     XDGDirs
+from Onboard.WindowUtils     import show_ask_string_dialog, \
+                                    show_confirmation_dialog
 
 
 app = "onboard"
@@ -165,12 +187,12 @@ class DialogBuilder(object):
             config_set_callback = self.bind_combobox_config_set
 
         w.set_active_id(id)
-        w.connect("changed", self.bind_combobox_callback,
+        w.connect("changed", self._bind_combobox_callback,
                   config_object, key, config_set_callback)
         getattr(config_object, key + '_notify_add')( \
-             lambda x: self.notify_combobox_callback(w, config_object, key,
+             lambda x: self._notify_combobox_callback(w, config_object, key,
                                                      config_get_callback))
-    def notify_combobox_callback(self, widget,
+    def _notify_combobox_callback(self, widget,
                                  config_object, key, config_get_callback):
         if config_get_callback:
             id = config_get_callback(config_object, key)
@@ -178,7 +200,7 @@ class DialogBuilder(object):
             id = str(getattr(config_object, key))
         widget.set_active_id(id)
 
-    def bind_combobox_callback(self, widget,
+    def _bind_combobox_callback(self, widget,
                                config_object, key, config_set_callback):
         id = widget.get_active_id()
         config_set_callback(config_object, key, id)
@@ -254,14 +276,11 @@ class Settings(DialogBuilder):
         self.icon_palette_toggle.set_active(config.icp.in_use)
         config.icp.in_use_notify_add(self.icon_palette_toggle.set_active)
 
-        #self.modeless_gksu_toggle = builder.get_object("modeless_gksu_toggle")
-        #self.modeless_gksu_toggle.set_active(config.modeless_gksu)
-        #config.modeless_gksu_notify_add(self.modeless_gksu_toggle.set_active)
-
-        self.onboard_xembed_toggle = builder.get_object("onboard_xembed_toggle")
+        self.onboard_xembed_toggle = \
+            builder.get_object("onboard_xembed_toggle")
         self.onboard_xembed_toggle.set_active(config.onboard_xembed_enabled)
-        config.onboard_xembed_enabled_notify_add( \
-                self.onboard_xembed_toggle.set_active)
+        config.onboard_xembed_enabled_notify_add(
+            self.onboard_xembed_toggle.set_active)
 
         self.show_tooltips_toggle = builder.get_object("show_tooltips_toggle")
         self.show_tooltips_toggle.set_active(config.show_tooltips)
@@ -271,20 +290,23 @@ class Settings(DialogBuilder):
         self.auto_show_toggle.set_active(config.auto_show.enabled)
         config.auto_show.enabled_notify_add(self.auto_show_toggle.set_active)
 
+        self.bind_combobox_id("status_icon_provider_combobox",
+                              config, "status_icon_provider")
         # window tab
         self.window_decoration_toggle = \
-                              builder.get_object("window_decoration_toggle")
-        self.window_decoration_toggle.set_active(config.window.window_decoration)
-        config.window.window_decoration_notify_add(lambda x:
-                                    [self.window_decoration_toggle.set_active(x),
-                                     self.update_window_widgets()])
+            builder.get_object("window_decoration_toggle")
+        self.window_decoration_toggle.set_active(
+            config.window.window_decoration)
+        config.window.window_decoration_notify_add(
+            lambda x: [self.window_decoration_toggle.set_active(x),
+                       self.update_window_widgets()])
 
         self.window_state_sticky_toggle = \
-                             builder.get_object("window_state_sticky_toggle")
-        self.window_state_sticky_toggle.set_active( \
-                                             config.window.window_state_sticky)
-        config.window.window_state_sticky_notify_add( \
-                                    self.window_state_sticky_toggle.set_active)
+            builder.get_object("window_state_sticky_toggle")
+        self.window_state_sticky_toggle.set_active(
+            config.window.window_state_sticky)
+        config.window.window_state_sticky_notify_add(
+            self.window_state_sticky_toggle.set_active)
 
         self.force_to_top_toggle = builder.get_object("force_to_top_toggle")
         self.force_to_top_toggle.set_active(config.is_force_to_top())
@@ -339,8 +361,16 @@ class Settings(DialogBuilder):
                         config.keyboard, "touch_feedback_enabled")
         self.bind_check("audio_feedback_enabled_toggle",
                         config.keyboard, "audio_feedback_enabled")
+        self.bind_check("audio_feedback_place_in_space_toggle",
+                        config.keyboard, "audio_feedback_place_in_space")
         self.bind_check("show_secondary_labels_toggle",
                         config.keyboard, "show_secondary_labels")
+        self.bind_check("upper_case_on_right_click_toggle",
+                        config.keyboard, "key_press_modifiers",
+                        config_get_callback=lambda co, key:
+                            co.can_upper_case_on_button(3),
+                        config_set_callback=lambda co, key, value:
+                            co.set_upper_case_on_button(3, value))
 
         # Keyboard - Advanced page
         self.bind_combobox_id("default_key_action_combobox",
@@ -362,6 +392,9 @@ class Settings(DialogBuilder):
         self.bind_spin("sticky_key_release_delay_spinbutton",
                             config.keyboard, "sticky_key_release_delay")
 
+        self.bind_spin("sticky_key_release_on_hide_delay_spinbutton",
+                            config.keyboard, "sticky_key_release_on_hide_delay")
+
         self.bind_combobox_id("touch_input_combobox",
                         config.keyboard, "touch_input")
 
@@ -554,17 +587,20 @@ class Settings(DialogBuilder):
             config.window.force_to_top = widget.get_active()
         self.update_window_widgets()
 
-    def on_keep_aspect_ratio_toggled(self,widget):
+    def on_keep_aspect_ratio_toggled(self, widget):
         config.window.keep_aspect_ratio = widget.get_active()
 
     def update_window_widgets(self):
-        force_to_top =  config.is_force_to_top()
+        force_to_top = config.is_force_to_top()
 
         w = self.wid("auto_show_settings_button")
         w.set_sensitive(config.is_auto_show_enabled())
 
-        self.icon_palette_toggle.set_sensitive( \
-                             not config.is_icon_palette_last_unhide_option())
+        w = self.wid("status_icon_provider_box")
+        w.set_sensitive(config.show_status_icon)
+
+        self.icon_palette_toggle.set_sensitive(
+            not config.is_icon_palette_last_unhide_option())
         active = config.is_icon_palette_in_use()
         if self.icon_palette_toggle.get_active() != active:
             self.icon_palette_toggle.set_active(active)
@@ -596,6 +632,8 @@ class Settings(DialogBuilder):
         if self.enable_inactive_transparency_toggle.get_active() != active:
             self.enable_inactive_transparency_toggle.set_active(active)
 
+        self.auto_show_toggle.set_active(config.auto_show.enabled)
+
     def update_all_widgets(self):
         pass
 
@@ -747,7 +785,7 @@ class Settings(DialogBuilder):
 
         self._add_layout_section(model, system, _("Core layouts"))
         self._add_layout_section(model, contributed, _("Contributions"))
-        self._add_layout_section(model, user, _("User layouts"))
+        self._add_layout_section(model, user, _("My layouts"))
 
         self.layout_view.expand_all()
         self.update_layout_view_selection()
@@ -1116,16 +1154,28 @@ class PageWordSuggestions(DialogBuilder):
         self.bind_check("language_button_toggle",
             config.word_suggestions, "wordlist_buttons",
             config_get_callback = lambda co, key: \
-                    config.word_suggestions.can_show_language_button(),
+                    co.can_show_language_button(),
             config_set_callback = lambda co, key, value: \
-                    config.word_suggestions.show_language_button(value))
+                    co.show_wordlist_button(co.KEY_ID_LANGUAGE, value))
 
         self.bind_check("pause_learning_button_toggle",
             config.word_suggestions, "wordlist_buttons",
             config_get_callback = lambda co, key: \
-                    config.word_suggestions.can_show_pause_learning_button(),
+                    co.can_show_pause_learning_button(),
             config_set_callback = lambda co, key, value: \
-                    config.word_suggestions.show_pause_learning_button(value))
+                    co.show_wordlist_button(co.KEY_ID_PAUSE_LEARNING, value))
+
+        def _get(co, key):
+            return co.can_show_more_predictions_button()
+
+        def _set(co, key, value):
+            co.show_wordlist_button(co.KEY_ID_NEXT_PREDICTIONS, value)
+            co.show_wordlist_button(co.KEY_ID_PREVIOUS_PREDICTIONS, value)
+
+        self.bind_check("more_predictions_button_toggle",
+                        config.word_suggestions, "wordlist_buttons",
+                        config_get_callback=_get,
+                        config_set_callback=_set)
 
         self.bind_combobox_id("learning_behavior_paused_combobox",
                         config.word_suggestions, "learning_behavior_paused")
@@ -1164,6 +1214,16 @@ class AutoShowDialog(DialogBuilder):
 
         DialogBuilder.__init__(self, builder)
 
+        def set_config(config_object, key, value):
+            self.bind_combobox_config_set(config_object, key, value)
+            self._update_ui()
+        self.bind_combobox_id("reposition_method_floating_combobox",
+                        config.auto_show, "reposition_method_floating",
+                        config_set_callback = set_config)
+        self.bind_combobox_id("reposition_method_docked_combobox",
+                        config.auto_show, "reposition_method_docked",
+                        config_set_callback = set_config)
+
         def set_config(config_object, key, value):
             setattr(config_object, key, value)
             self._update_ui()
@@ -1195,6 +1255,11 @@ class AutoShowDialog(DialogBuilder):
         w = self.wid("hide_on_key_press_box")
         w.set_sensitive(config.is_auto_hide_enabled())
 
+        docked = config.is_docking_enabled()
+        w = self.wid("reposition_method_floating_box")
+        w.set_visible(not docked)
+        w = self.wid("reposition_method_docked_box")
+        w.set_visible(docked)
 
 class DockingDialog(DialogBuilder):
     """ Dialog "Docking Settings" """
@@ -1409,14 +1474,14 @@ class ThemeDialog(DialogBuilder):
                     for font in font_families]
 
         families.sort(key=lambda x: x[0])
-        families = [(_("Default"), "Normal"),
+        families = [(_("Default"), ""),
                     ("-", "-")] + families
         fd = Pango.FontDescription(self.theme.key_label_font)
         family = fd.get_family()
         for f in families:
             it = self.font_list.append(f)
             if  f[1] == family or \
-               (f[1] == "Normal" and not family):
+               (f[1] == "" and not family):
                 self.font_combobox.set_active_iter(it)
 
     def font_combobox_row_separator_func(self, model, iter, data):
@@ -1891,9 +1956,9 @@ class CellRendererMapping(Gtk.CellRendererText):
                         str('pointer-mode') : (bool, '', '', True,
                                              GObject.PARAM_READWRITE) }
 
-    __gsignals__ = { str('mapping-edited') : (GObject.SIGNAL_RUN_LAST,
+    __gsignals__ = { str('mapping-edited') : (GObject.SignalFlags.RUN_LAST,
                                              None, (str, int, bool)),
-                     str('mapping-cleared'): (GObject.SIGNAL_RUN_LAST,
+                     str('mapping-cleared'): (GObject.SignalFlags.RUN_LAST,
                                              None, (str, bool)) }
 
     def __init__(self):
